# encoding: utf-8

module Rumid

  # Abstract class, representing Request. Should be extended to specify protocol.
  # @abstract
  # @example
  #   class Foo < Request
  #     read_wire do |io|
  #       io.readline
  #     end
  #
  #     format_request do |raw|
  #       {route: raw[1], data: raw[2]}
  #     end
  #
  #     validate_request do |raw|
  #       raw[4] != raw[5]
  #     end
  #
  #     validate_request do |raw|
  #       raw[1]+raw[2] != raw[3]
  #     end
  #   end
  class Request
    include Errors
    # Raw data from IO getter block
    # @note Proc requires [IO]
    @@wire_handler = nil
    # Raw data to request hash parser.
    # @note Proc requires [String] raw data
    @@wire_parser = nil
    # [Array(Proc)] validation procs array
    # @note Proc requires [String] raw data
    @@request_validators = []

    # Initalize raw data from IO getter
    # @param block [Proc] representing raw data from IO getter
    # @note block requires requires [IO]
    def self.read_wire(&block)
      @@wire_handler = block
    end

    # Initialize raw data to request hash parser
    # @param block [Proc] representing raw data to request hash parser
    # @note block requires requires [String] raw data
    def self.format_request(&block)
      @@wire_parser = block
    end

    # Method gets only valid requests
    # @param io [IO] representing raw data to request hash parser
    # @note Typicaly you should not call this method
    def get_request(io)
      raw = @@wire_handler.call(io)
      throw NotValidRequestError if @@request_validators.all?{|block| block.call(raw)}
      @@wire_parser.call(raw)
    end

    # Initialize new request validator
    # @param block [Proc] represent request validator
    # @note Proc requires [Stirng] raw data
    # @note Proc returns [Boolean] request validness
    def self.validate_request(&block)
      @@request_validators << block
    end
  end
end
