# encoding: utf-8

module Rumid

  # Abstract class, representing Server. Should be extended to specify protocol.
  # @abstract
  # @example
  #   class Foo < Server
  #     handle_request_with(BarRequest)
  #
  #     wrap_response do |response|
  #       "Responce is: #{responce[:route]}|#{responce[:data]}"
  #     end
  #
  #     route "foo" do |request|
  #       {route: "foo", data: 1}
  #     end
  #
  #     route "bar" do |request|
  #       {route: "bar", data: 2}
  #     end
  #   end
  class Server
    include Errors

    # @!constant routes [{String=>Proc}] representing routes strings and procs to handle requests
    # @see Server.route
    @@routes = {}

    # @!constant request_handler [Request] klass, backed to handle this protocol requests
    # @see Rumid::Request
    @@request_handler = nil

    def initialize
      @responsed = false
    end


    # Main server function. Do all work from getting raw request to sending raw response
    # @note Typicaly you should not call this method
    # @param io [IO] I/O stream for getting requests and sending responces
    def serve(io)
      @io = io
      request_hash = get_request
      route_handle(request_hash)
    end

    # @return [Hash] hash with Keys - routes, Values - Procs for handling requests
    def self.show_routes
      @@routes
    end

    private

    # Method reading data from I/O stream
    # @note Typicaly you should not call this method
    def get_request
      raise NotImplementedError, "Request handler is not initialized." if @@request_handler.nil?
      @@request_handler.get_request(@io)
    end

    # Method which sends request direct roated handler
    # @note Typicaly you should not call this method
    # @param request_hash [{Symbol=>Proc}] request hash
    def route_handle(request_hash)
      @@routes.each do |route,method|
        return response_with(@@response_wrapper.call(method.call(request_hash))) unless (route =~ request_hash[:route]).nil?
      end
      warn "This request doesn't match any route."
    end


    # Initialising new route
    # @param route [String] representing route string
    # @param block [Proc] representing route handler
    # @example
    #   route "foo" do |request|
    #     {route: "foo", data: 1}
    #   end
    # @note Proc requires [{Symbol=>String}] request hash
    # @note Proc returns [{Symbol=>String}] response hash
    def self.route(route, &block)
      @@routes[Regexp.new(route)] = block
    end

    # Initializing how to pack responces to send back by protocol
    # @param block [Proc] representing respose wrapper
    # @example
    #   wrap_response do |response|
    #     "Responce is: #{responce[:route]}|#{responce[:data]}"
    #   end
    # @note Proc requires [{Symbol=>String}] response hash
    # @note Proc returns [String] raw packet
    def self.wrap_response(&block)
      @@response_wrapper = block
    end

    # Initializing [Request] class to handle requests
    # @param request_handler_class [Class] klass representing Request specified by protocol
    # @example
    #   handle_request_with FooRequest
    # @note request_handler_class extends [Request]
    def self.handle_request_with(request_handler_class)
      @@request_handler = request_handler_class.new
    end

    # Method sending response. Checks response uniquness
    # @param response [String] response raw string
    def response_with(response)
      if @responsed == true then
        warn "Trying to response with:\n  #{response}\nfor already responsed request!"
        return
      end
      @io.write(response)
      @responsed = true
    end
  end
end
